package com.gic.haoban.manage.service.service.chat.impl;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.alibaba.fastjson.JSON;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.commons.util.UniqueIdUtils;
import com.gic.haoban.manage.api.dto.SecretSettingDTO;
import com.gic.haoban.manage.api.dto.WxEnterpriseQwDTO;
import com.gic.haoban.manage.api.enums.SecretTypeEnum;
import com.gic.haoban.manage.service.dao.mapper.StaffMapper;
import com.gic.haoban.manage.service.dao.mapper.chat.GroupChatMapper;
import com.gic.haoban.manage.service.dao.mapper.chat.GroupChatUserMapper;
import com.gic.haoban.manage.service.entity.TabHaobanExternalClerkRelated;
import com.gic.haoban.manage.service.entity.TabHaobanStaff;
import com.gic.haoban.manage.service.entity.chat.TabGroupChat;
import com.gic.haoban.manage.service.entity.chat.TabGroupChatUser;
import com.gic.haoban.manage.service.service.ExternalClerkRelatedService;
import com.gic.haoban.manage.service.service.SecretSettingService;
import com.gic.haoban.manage.service.service.WxEnterpriseService;
import com.gic.haoban.manage.service.service.chat.GroupChatService;
import com.gic.member.api.dto.MemberDTO;
import com.gic.member.api.service.MemberService;
import com.gic.wechat.api.dto.qywx.chat.GroupChatDetailDTO;
import com.gic.wechat.api.dto.qywx.chat.GroupChatDetailMemberDTO;
import com.gic.wechat.api.dto.qywx.chat.GroupChatListDTO;
import com.gic.wechat.api.dto.qywx.chat.GroupChatListItemDTO;
import com.gic.wechat.api.service.qywx.QywxChatApiService;

@Service
public class GroupChatServiceImpl implements GroupChatService {

    private static final Logger logger = LoggerFactory.getLogger(GroupChatService.class);

    @Autowired
    private QywxChatApiService qywxChatApiService;
    @Autowired
    private GroupChatMapper groupChatMapper;
    @Autowired
    private GroupChatUserMapper groupChatUserMapper;
    @Autowired
    private WxEnterpriseService wxEnterpriseService;
    @Autowired
    private MemberService memberService;
    @Autowired
    private ExternalClerkRelatedService externalClerkRelatedService;
    @Autowired
    private StaffMapper staffMapper;
    @Autowired
    private SecretSettingService secretSettingService;

    public void initGroupChat(String wxEnterpriseId) {
        WxEnterpriseQwDTO qwDTO = this.wxEnterpriseService.getQwInfo(wxEnterpriseId);
        if (null == qwDTO) {
            logger.info("wxEnterpriseId不存在,id={}", wxEnterpriseId);
            return;
        }
        int pageNum = 0;
        int pageSize = 10;
        while (true) {
            List<TabHaobanStaff> staffList = this.staffMapper.listAllByPage(wxEnterpriseId, pageNum * pageSize,
                    pageSize);
            if (CollectionUtils.isEmpty(staffList)) {
                break;
            }
            logger.info("查询staff表第{}页,共{}条", pageNum + 1, staffList.size());
            List<String> userIdList = null;
            if (qwDTO.needOpenUserId3th()) {
                userIdList = staffList.stream().filter(dto -> StringUtils.isNotBlank(dto.getWxOpenUseId()))
                        .map(dto -> dto.getWxOpenUseId()).collect(Collectors.toList());
            } else {
                userIdList = staffList.stream().filter(dto -> StringUtils.isNotBlank(dto.getWxUserId()))
                        .map(dto -> dto.getWxUserId()).collect(Collectors.toList());
            }
            if (CollectionUtils.isNotEmpty(userIdList)) {
                this.saveGroupChat(qwDTO, userIdList, null);
            }
            pageNum++;
        }
    }

    public void saveGroupChat(WxEnterpriseQwDTO qwDTO, List<String> userIdList, String cursor) {
        logger.info("qwDTO={},idlist={},cursor={}", JSON.toJSONString(qwDTO), JSON.toJSONString(userIdList), cursor);
        int limit = 1;
        SecretSettingDTO secretSetting = secretSettingService.getSecretSetting(qwDTO.getWxEnterpriseId(),
                SecretTypeEnum.CUSTOMIZED_APP.getVal());
        if (null == secretSetting) {
            logger.info("没有配置代开应用");
        }
        String secret = secretSetting.getSecretVal();
        ServiceResponse<GroupChatListDTO> resp = this.qywxChatApiService.groupchatList(qwDTO.getThirdCorpid(), secret,
                userIdList, limit, cursor);
        if (!resp.isSuccess()) {
            logger.info("拉取群异常={}", JSON.toJSON(resp));
            return;
        }
        GroupChatListDTO dto = resp.getResult();
        cursor = dto.getNext_cursor();
        List<GroupChatListItemDTO> list = dto.getGroup_chat_list();
        for (GroupChatListItemDTO item : list) {
            String wxChatId = item.getChat_id();
            TabGroupChat groupChat = this.groupChatMapper.selectByWxChatId(wxChatId);
            Long groupChatId = null;
            if (null != groupChat) {
                groupChatId = groupChat.getGroupChatId();
                logger.info("群已存在,chatid={},groupChatId={}", wxChatId, groupChatId);
            } else {
                groupChatId = UniqueIdUtils.uniqueLong();
                TabGroupChat chat = new TabGroupChat();
                chat.setGroupChatId(groupChatId);
                chat.setWxEnterpriseId(qwDTO.getWxEnterpriseId());
                chat.setWxChatId(wxChatId);
                chat.setChatStatus(item.getStatus());
                chat.setCreateTime(new Date());
                chat.setUpdateTime(new Date());
                chat.setStatusFlag(0);
                chat.setDeleteFlag(0);
                chat.setTotalCount(0);
                chat.setTotalMemberCount(0);
                chat.setAddMemberCount(0);
                chat.setAddCount(0);
                chat.setQuitCount(0);
                chat.setQuitMemberCount(0);
                this.groupChatMapper.insert(chat);
            }
            this.groupChatDetailInit(groupChatId);
        }
        if (org.apache.commons.lang3.StringUtils.isNotBlank(cursor)) {
            this.saveGroupChat(qwDTO, userIdList, cursor);
        }
    }

    public void groupChatDetailInit(Long groupChatId) {
        TabGroupChat chat = this.groupChatMapper.selectById(groupChatId);
        String wxEnterpriseId = chat.getWxEnterpriseId();
        String enterpriseId = chat.getEnterpriseId();
        WxEnterpriseQwDTO qwDTO = this.wxEnterpriseService.getQwInfo(wxEnterpriseId);
        SecretSettingDTO secretSetting = secretSettingService.getSecretSetting(qwDTO.getWxEnterpriseId(),
                SecretTypeEnum.CUSTOMIZED_APP.getVal());
        if (null == secretSetting) {
            logger.info("没有配置代开应用");
        }
        String secret = secretSetting.getSecretVal();
        ServiceResponse<GroupChatDetailDTO> detailResp = this.qywxChatApiService.groupchatDetail(qwDTO.getThirdCorpid(),
                secret, chat.getWxChatId(), 1);

        GroupChatDetailDTO detail = detailResp.getResult();
        chat.setGroupChatId(groupChatId);
        chat.setChatAddTime(new Date(detail.getCreate_time() * 1000));
        chat.setChatNotice(detail.getNotice());
        // 处理群主
        String owner = detail.getOwner();
        TabHaobanStaff staff = this.staffMapper.selectByUserIdAndEnterpriseId(owner, wxEnterpriseId);
        if (null == staff) {
            logger.info("群主不存在,userid={}", owner);
            return;
        }
        chat.setStaffId(staff.getStaffId());
        logger.info("更新群信息={}", JSON.toJSONString(chat));
        this.groupChatMapper.updateByPrimaryKey(chat);

        List<GroupChatDetailMemberDTO> memberList = detail.getMember_list();
        List<TabGroupChatUser> userList = new ArrayList<>();
        for (GroupChatDetailMemberDTO member : memberList) {
            String userId = member.getUserid();
            TabGroupChatUser user = this.groupChatUserMapper.selectByUserIdAndChatId(groupChatId, userId);
            if (null != user) {
                logger.info("更新用户");
                this.matchGicMember(wxEnterpriseId, enterpriseId, member.getUnionid(), userId, user);
                this.groupChatUserMapper.updateByPrimaryKey(user);
                continue;
            }
            user = new TabGroupChatUser();
            user.setChatUserId(UniqueIdUtils.uniqueLong());
            user.setWxEnterpriseId(wxEnterpriseId);
            user.setEnterpriseId(enterpriseId);
            user.setUserType(member.getType());
            user.setJoinTime(new Date(member.getJoin_time() * 1000));
            user.setUserName(member.getName());
            user.setNickName(member.getGroup_nickname());
            user.setGroupChatId(groupChatId);
            user.setUserId(userId);
            user.setJoinScene(member.getJoin_scene());
            user.setUnionid(member.getUnionid());
            user.setDeleteFlag(0);
            user.setStatusFlag(1);
            user.setCreateTime(new Date());
            user.setUpdateTime(new Date());
            if (null != member.getInvitor()) {
                user.setInvitorUserId(member.getInvitor().getUserid());
            }
            this.matchGicMember(wxEnterpriseId, enterpriseId, member.getUnionid(), userId, user);
            userList.add(user);
        }
        for (TabGroupChatUser item : userList) {
            this.groupChatUserMapper.insert(item);
        }
    }

    private void matchGicMember(String wxEnterpriseId, String enterpriseId, String unionid, String externalUserId,
            TabGroupChatUser user) {
        if (StringUtils.isBlank(enterpriseId)) {
            return;
        }
        if (StringUtils.isNotBlank(unionid)) {
            MemberDTO member = this.memberService.getMemberByUnionid(unionid, enterpriseId);
            if (null != member) {
                user.setMemberId(member.getMemberId());
                user.setUserType(3);
            }
        } else if (StringUtils.isNotBlank(externalUserId)) {
            List<TabHaobanExternalClerkRelated> list = this.externalClerkRelatedService
                    .listExternalByExternalUserid(wxEnterpriseId, externalUserId);
            list = list.stream().filter(dto -> StringUtils.isNotBlank(dto.getMemberId())).collect(Collectors.toList());
            if (CollectionUtils.isNotEmpty(list)) {
                user.setMemberId(list.get(0).getMemberId());
                user.setUserType(3);
            }
        }
    }

}